@@ -30,6 +30,7 @@ from openstackclient.i18n import _
 from openstackclient.identity import common
 
 
+CONTAINER_CHOICES = ["ami", "ari", "aki", "bare", "docker", "ova", "ovf"]
 DEFAULT_CONTAINER_FORMAT = 'bare'
 DEFAULT_DISK_FORMAT = 'raw'
 DISK_CHOICES = ["ami", "ari", "aki", "vhd", "vmdk", "raw", "qcow2", "vhdx",
@@ -135,9 +136,13 @@ class CreateImage(command.ShowOne):
         parser.add_argument(
             "--container-format",
             default=DEFAULT_CONTAINER_FORMAT,
+            choices=CONTAINER_CHOICES,
             metavar="<container-format>",
-            help=_("Image container format "
-                   "(default: %s)") % DEFAULT_CONTAINER_FORMAT,
+            help=(_("Image container format. "
+                    "The supported options are: %(option_list)s. "
+                    "The default format is: %(default_opt)s") %
+                  {'option_list': ', '.join(CONTAINER_CHOICES),
+                   'default_opt': DEFAULT_CONTAINER_FORMAT})
         )
         parser.add_argument(
             "--disk-format",
@@ -659,8 +664,9 @@ class SetImage(command.Command):
         parser.add_argument(
             "--container-format",
             metavar="<container-format>",
-            help=_("Image container format "
-                   "(default: %s)") % DEFAULT_CONTAINER_FORMAT,
+            choices=CONTAINER_CHOICES,
+            help=_("Image container format. The supported options are: %s") %
+            ', '.join(CONTAINER_CHOICES)
         )
         parser.add_argument(
             "--disk-format",
@@ -777,6 +783,23 @@ class SetImage(command.Command):
                 dest=deadopt.replace('-', '_'),
                 help=argparse.SUPPRESS,
             )
+
+        membership_group = parser.add_mutually_exclusive_group()
+        membership_group.add_argument(
+            "--accept",
+            action="store_true",
+            help=_("Accept the image membership"),
+        )
+        membership_group.add_argument(
+            "--reject",
+            action="store_true",
+            help=_("Reject the image membership"),
+        )
+        membership_group.add_argument(
+            "--pending",
+            action="store_true",
+            help=_("Reset the image membership to 'pending'"),
+        )
         return parser
 
     def take_action(self, parsed_args):
@@ -828,12 +851,14 @@ class SetImage(command.Command):
             project_arg = parsed_args.owner
             LOG.warning(_('The --owner option is deprecated, '
                           'please use --project instead.'))
+        project_id = None
         if project_arg:
-            kwargs['owner'] = common.find_project(
+            project_id = common.find_project(
                 identity_client,
                 project_arg,
                 parsed_args.project_domain,
             ).id
+            kwargs['owner'] = project_id
 
         image = utils.find_resource(
             image_client.images, parsed_args.image)
@@ -846,6 +871,21 @@ class SetImage(command.Command):
             image_client.images.reactivate(image.id)
             activation_status = "activated"
 
+        membership_group_args = ('accept', 'reject', 'pending')
+        membership_status = [status for status in membership_group_args
+                             if getattr(parsed_args, status)]
+        if membership_status:
+            # If a specific project is not passed, assume we want to update
+            # our own membership
+            if not project_id:
+                project_id = self.app.client_manager.auth_ref.project_id
+            # The mutually exclusive group of the arg parser ensure we have at
+            # most one item in the membership_status list.
+            if membership_status[0] != 'pending':
+                membership_status[0] += 'ed'  # Glance expects the past form
+            image_client.image_members.update(
+                image.id, project_id, membership_status[0])
+
         if parsed_args.tags:
             # Tags should be extended, but duplicates removed
             kwargs['tags'] = list(set(image.tags).union(set(parsed_args.tags)))
